<?php

namespace Drupal\graphql_webform\Plugin\GraphQL\DataProducer;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\GraphQL\Execution\FieldContext;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Load a webform by ID.
 *
 * @DataProducer(
 *   id = "webform_load",
 *   name = @Translation("Webform Load"),
 *   description = @Translation("Load a webform by ID."),
 *   produces = @ContextDefinition("entity:webform",
 *     label = @Translation("Webform"),
 *     required = FALSE
 *   ),
 *   consumes = {
 *     "id" = @ContextDefinition("string",
 *       label = @Translation("Webform ID")
 *     ),
 *     "sourceEntityType" = @ContextDefinition("string",
 *       label = @Translation("Source Entity Type"),
 *       required = FALSE
 *     ),
 *     "sourceEntityId" = @ContextDefinition("string",
 *       label = @Translation("Source Entity ID"),
 *       required = FALSE
 *     )
 *   }
 * )
 */
class WebformLoad extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * Webform entity storage.
   *
   * @var \Drupal\webform\WebformEntityStorage
   */
  protected $webformStorage;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $pluginId,
    $pluginDefinition
  ) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('entity_type.manager')
    );
  }

  /**
   * NodeLogoTheme constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity Type Manager.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    EntityTypeManagerInterface $entityTypeManager
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->webformStorage = $entityTypeManager->getStorage('webform');
  }

  /**
   * Resolver.
   */
  public function resolve(string $id, $sourceEntityType, $sourceEntityId, FieldContext $field) {
    /** @var \Drupal\webform\Entity\Webform|null $webform */
    $webform = $this->webformStorage->load($id);
    if ($webform && $webform->access('view')) {
      if ($sourceEntityType && $sourceEntityId) {
        $field->setContextValue('webform_source_entity_type', $sourceEntityType);
        $field->setContextValue('webform_source_entity_id', $sourceEntityId);
      }
      return $webform;
    }
  }

}
