<?php

namespace Drupal\graphql_webform\Plugin\GraphQL\DataProducer;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Return the webform submission.
 *
 * @DataProducer(
 *   id = "webform_load_submission",
 *   name = @Translation("Webform Load Submission"),
 *   description = @Translation("Returns the webform confirmation."),
 *   produces = @ContextDefinition("entity:webform_submission",
 *     label = @Translation("Webform Submission"),
 *     required = FALSE
 *   ),
 *   consumes = {
 *     "id" = @ContextDefinition("string",
 *       label = @Translation("Webform Submission ID")
 *     ),
 *     "token" = @ContextDefinition("string",
 *       label = @Translation("Webform Submission Token")
 *     ),
 *   }
 * )
 */
class WebformLoadSubmission extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The renderer service.
   *
   * @var \Drupal\webform\WebformSubmissionStorage
   */
  protected $submissionStorage;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $pluginId,
    $pluginDefinition
  ) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('entity_type.manager')
    );
  }

  /**
   * WebformSubmissionConfirmation constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity Type Manager.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    EntityTypeManagerInterface $entityTypeManager
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->submissionStorage = $entityTypeManager->getStorage('webform_submission');
  }

  /**
   * Resolver.
   */
  public function resolve(string $id, string $token) {
    /** @var \Drupal\webform\Entity\WebformSubmission|null $submission */
    $submission = $this->submissionStorage->load($id);
    if (!$submission) {
      return NULL;
    }

    $submissionToken = $submission->getToken();

    if ($submissionToken !== $token) {
      return NULL;
    }

    return $submission;
  }

}
