<?php

declare(strict_types=1);

namespace Drupal\graphql_webform;

use Drupal\graphql_core_schema\EntitySchemaHelper;
use Drupal\webform\Element\WebformElementStates;
use Drupal\webform\Entity\Webform;
use Drupal\webform\Plugin\WebformElement\ContainerBase;
use Drupal\webform\Plugin\WebformElement\DateBase;
use Drupal\webform\Plugin\WebformElement\OptionsBase;
use Drupal\webform\Plugin\WebformElement\TextBase;
use Drupal\webform\Plugin\WebformElement\WebformManagedFileBase;
use Drupal\webform\Plugin\WebformElementInterface;
use Drupal\webform\Plugin\WebformElementOtherInterface;
use GraphQL\Type\Definition\EnumType;
use GraphQL\Type\Definition\InterfaceType;
use GraphQL\Type\Definition\ObjectType;
use GraphQL\Type\Definition\Type;

/**
 *
 */
class WebformSchemaBuilder {

  public const WEBFORM_STATES = [
    'checked',
    'collapsed',
    'disabled',
    'enabled',
    'expanded',
    'invisible',
    'invisibleSlide',
    'optional',
    'readonly',
    'readwrite',
    'required',
    'unchecked',
    'visible',
    'visibleSlide',
  ];

  public const WEBFORM_TRIGGERS = [
    'empty' => 'EMPTY',
    'filled' => 'FILLED',
    'checked' => 'CHECKED',
    'unchecked' => 'UNCHECKED',
    'value' => 'VALUE_IS',
    '!value' => 'VALUE_IS_NOT',
    'pattern' => 'PATTERN',
    '!pattern' => 'NOT_PATTERN',
    'less' => 'LESS_THAN',
    'less_equal' => 'LESS_THAN_EQUAL_TO',
    'greater' => 'GREATER_THAN',
    'greater_equal' => 'GREATER_THAN_EQUAL_TO',
    'between' => 'BETWEEN',
    '!between' => 'NOT_BETWEEN',
  ];

  /**
   * The generated types.
   *
   * @var \GraphQL\Type\Definition\TypeWithFields[]
   */
  protected $types;

  /**
   * The WebformElement interface fields.
   *
   * @var array
   */
  protected $element;

  /**
   * The WebformElementContainerBase interface fields.
   *
   * @var array
   */
  protected $elementContainer;

  /**
   * The WebformElementOptions interface fields.
   *
   * @var array
   */
  protected $elementOptions;

  /**
   * The WebformElementOtherBase interface fields.
   *
   * @var array
   */
  protected $elementOtherBase;

  /**
   * The WebformElementDateBase interface fields.
   *
   * @var array
   */
  protected $elementDateBase;

  /**
   * The WebformElementManagedFileBase interface fields.
   *
   * @var array
   */
  protected $elementManagedFileBase;

  /**
   * The WebformElementTextBase interface fields.
   *
   * @var array
   */
  protected $elementTextBase;

  /**
   *
   */
  public function __construct() {
    $this->types = [];

    $this->types['WebformFormItem'] = new ObjectType([
      'name' => 'WebformFormItem',
      'fields' => [
        'key' => $this->type(Type::nonNull(Type::string()), 'The key of the element.'),
        'name' => $this->type(Type::nonNull(Type::string()), 'The name of the element.'),
        'type' => $this->type(Type::nonNull(Type::string()), "Array of names of all the element's parents (including itself) in the render array. See also parents, tree."),
        'title' => $this->type(Type::string(), ''),
        'help' => $this->type(Type::string(), ''),
        'helpTitle' => $this->type(Type::string(), ''),
        'helpDisplay' => $this->type(Type::string(), ''),
        'more' => $this->type(Type::string(), ''),
        'moreTitle' => $this->type(Type::string(), ''),
        'titleDisplay' => $this->type(Type::string(), 'Where and how to display the #title. Possible values:
  - before: Label goes before the element (default for most elements).
  - after: Label goes after the element (default for radio elements).
  - invisible: Label is there but is made invisible using CSS.
  - attribute: Make it the title attribute (hover tooltip).'),
        'defaultValue' => $this->type(Type::string(), 'Default value for the element. See also value.'),
        'description' => $this->type(Type::string(), ''),
        'descriptionDisplay' => $this->type(Type::string(), ''),
        'disabled' => $this->type(Type::boolean(), 'If TRUE, the element is shown but does not accept user input.'),
        'readonly' => $this->type(Type::boolean(), 'Make this element non-editable with the user entered (e.g. via developer tools) value <strong>submitted</strong>. Useful for displaying default value. Changeable via JavaScript.'),
        'fieldPrefix' => $this->type(Type::string(), 'Prefix to display before the HTML input element. Should be translated, normally. If it is not already wrapped in a safe markup object, will be filtered for XSS safety. Note that the contents of this prefix are wrapped in a <span> element, so the value should not contain block level HTML. Any HTML added must be valid, i.e. any tags introduced inside this prefix must also be terminated within the prefix.'),
        'fieldSuffix' => $this->type(Type::string(), 'Suffix to display after the HTML input element. Should be translated, normally. If it is not already wrapped in a safe markup object, will be filtered for XSS safety. Note that the contents of this suffix are wrapped in a <span> element, so the value should not contain block level HTML. Any HTML must also be valid, i.e. any tags introduce inside this suffix must also be terminated within the suffix.'),
        'hasGarbageValue' => $this->type(Type::boolean(), 'Internal only. Set to TRUE to indicate that the value property of an element should not be used or processed.'),
        'input' => $this->type(Type::boolean(), 'Whether or not the element accepts input.'),
        'parents' => $this->type(Type::listOf(Type::string()), "Array of names of the element's parents for purposes of getting values out of form_state. See also #array_parents, tree."),
        'processed' => $this->type(Type::boolean(), 'Set to TRUE when the element is processed.'),
        'required' => $this->type(Type::boolean(), 'Whether or not input is required on the element.'),
        'requiredError' => $this->type(Type::string(), ''),
        'tree' => $this->type(Type::boolean(), 'TRUE if the values of this element and its children should be hierarchical in $form_state; FALSE if the values should be flat. See also parents, arrayParents.'),
        'flex' => $this->type(Type::int(), 'The flex property specifies the length of the item, relative to the rest of the flexible items inside the same container.'),
        'states' => fn() => $this->types['WebformElementStates'],
        'element' => fn() => $this->type($this->types['WebformElement'], 'The webform element plugin.'),
      ],
    ]);

    $this->element = [
      'type' => Type::string(),
    ];

    $this->elementContainer = [
      'items' => fn () => Type::nonNull(Type::listOf(Type::nonNull($this->types['WebformFormItem']))),
    ];
    $this->types['WebformElement'] = new InterfaceType([
      'name' => 'WebformElement',
      'fields' => $this->element,
    ]);
    $this->types['WebformElementContainerBase'] = new InterfaceType([
      'name' => 'WebformElementContainerBase',
      'fields' => $this->elementContainer,
    ]);
    $this->types['WebformForm'] = new ObjectType([
      'name' => 'WebformForm',
      'fields' => [
        'items' => fn() => Type::listOf(Type::nonNull($this->types['WebformFormItem'])),
      ],
    ]);

    $this->elementOptions = [
      'options' => fn () => Type::listOf($this->types['WebformOptionsOption']),
      'optionsRandomize' => Type::boolean(),
    ];

    $this->types['WebformOptionsOption'] = new ObjectType([
      'name' => 'WebformOptionsOption',
      'fields' => [
        'value' => Type::nonNull(Type::string()),
        'description' => Type::string(),
        'label' => Type::nonNull(Type::string()),
      ],
    ]);

    $this->types['WebformElementOptionsBase'] = new InterfaceType([
      'name' => 'WebformElementOptionsBase',
      'fields' => $this->elementOptions,
    ]);

    $this->elementOtherBase = [
      'other' => fn () => $this->types['WebformFormItem'],
    ];

    $this->types['WebformElementOtherBase'] = new InterfaceType([
      'name' => 'WebformElementOtherBase',
      'fields' => $this->elementOtherBase,
    ]);

    $this->elementDateBase = [
      'dateDateMin' => Type::string(),
      'dateDateMax' => Type::string(),
      'dateDays' => Type::listOf(Type::string()),
    ];

    $this->types['WebformElementDateBase'] = new InterfaceType([
      'name' => 'WebformElementDateBase',
      'fields' => $this->elementDateBase,
    ]);

    $this->elementManagedFileBase = [
      'maxFilesize' => Type::string(),
      'fileExtensions' => Type::string(),
      'fileName' => Type::string(),
      'fileHelp' => Type::string(),
      'filePreview' => Type::string(),
      'filePlaceholder' => Type::string(),
      'button' => Type::boolean(),
      'buttonTitle' => Type::string(),
      'cardinality' => Type::int(),
    ];

    $this->types['WebformElementManagedFileBase'] = new InterfaceType([
      'name' => 'WebformElementManagedFileBase',
      'fields' => $this->elementManagedFileBase,
    ]);

    $this->elementTextBase = [
      'readonly' => Type::boolean(),
      'size' => Type::int(),
      'minlength' => Type::int(),
      'maxlength' => Type::int(),
      'placeholder' => Type::string(),
      'autocomplete' => Type::string(),
      'pattern' => Type::string(),
      'patternError' => Type::string(),
    ];

    $this->types['WebformElementTextBase'] = new InterfaceType([
      'name' => 'WebformElementTextBase',
      'fields' => $this->elementTextBase,
    ]);

    $this->types['WebformStateLogic'] = new EnumType([
      'name' => 'WebformStateLogic',
    ]);

    $this->types['WebformStateLogic'] = new EnumType([
      'name' => 'WebformStateLogic',
      'values' => [
        'AND' => 'and',
        'OR' => 'or',
        'XOR' => 'xor',
      ],
    ]);

    $elementStates = WebformElementStates::getTriggerOptions();
    $triggersValues = [];

    foreach ($elementStates as $id => $label) {
      $triggersValues[self::WEBFORM_TRIGGERS[$id]] = [
        'value' => $id,
        'description' => (string) $label,
      ];
    }
    $this->types['WebformStateConditionTrigger'] = new EnumType([
      'name' => 'WebformStateConditionTrigger',
      'values' => $triggersValues,
    ]);

    $this->types['WebformElementState'] = new ObjectType([
      'name' => 'WebformElementState',
      'fields' => [
        'conditions' => fn () => Type::listOf($this->types['WebformElementStateCondition']),
        'logic' => fn () => $this->types['WebformStateLogic'],
      ],
    ]);

    $this->types['WebformElementStateCondition'] = new ObjectType([
      'name' => 'WebformElementStateCondition',
      'fields' => [
        'field' => Type::string(),
        'fieldValue' => Type::string(),
        'selector' => Type::string(),
        'value' => Type::string(),
        'trigger' => fn () => $this->types['WebformStateConditionTrigger'],
      ],
    ]);

    $stateOptions = WebformElementStates::getStateOptions();
    $stateOptionsMapped = [];
    foreach ($stateOptions as $group => $states) {
      foreach ($states as $state => $label) {
        $stateOptionsMapped[$state] = $group . ': ' . $label;
      }
    }
    $stateFields = [];
    foreach (self::WEBFORM_STATES as $state) {
      $key = str_replace('_', '-', EntitySchemaHelper::toSnakeCase($state));
      $stateFields[$state] = fn() => $this->type($this->types['WebformElementState'], $stateOptionsMapped[$key]);
    }
    $this->types['WebformElementStates'] = new ObjectType([
      'name' => 'WebformElementStates',
      'fields' => $stateFields,
    ]);

    $this->createWebformSettingsType();
  }

  /**
   *
   */
  protected function createWebformSettingsType() {
    $default_settings = Webform::getDefaultSettings();

    // Some settings have a default value of NULL, so we need to map these
    // manually.
    $null_settings = [
      'ajax_speed' => 'integer',
      'limit_total' => 'integer',
      'limit_total_interval' => 'integer',
      'limit_user' => 'integer',
      'limit_user_interval' => 'integer',
      'entity_limit_total' => 'integer',
      'entity_limit_total_interval' => 'integer',
      'entity_limit_user' => 'integer',
      'entity_limit_user_interval' => 'integer',
      'purge_days' => 'integer',
    ];

    $fields = [];

    foreach ($default_settings as $prop => $value) {
      $type = $null_settings[$prop] ?? $value;
      $graphqlType = $this->getType($type);
      if ($graphqlType) {
        $name = EntitySchemaHelper::toCamelCase($prop);
        $fields[$name] = $graphqlType;
      }
    }

    $this->types['WebformSettings'] = new ObjectType([
      'name' => 'WebformSettings',
      'fields' => $fields,
    ]);
  }

  /**
   * Get the GraphQL type for a PHP type.
   *
   * @var mixed $value
   *   The value to get the type for.
   *
   * @return string|null
   *   The GraphQL type.
   */
  protected function getType($value) {
    switch (gettype($value)) {
      case 'boolean':
        return Type::boolean();

      case 'integer':
        return Type::int();

      case 'double':
        return Type::float();

      case 'string':
        return Type::string();
    }

    return NULL;
  }

  /**
   * Generate the types for the form elements.
   */
  public function generateElementType($id, $label, $description, ?WebformElementInterface $plugin = NULL) {
    $interfaces = [
      fn () => $this->types['WebformElement'],
    ];
    $fields = [
      ...$this->element,
    ];

    if ($plugin instanceof ContainerBase || ($plugin && $plugin->isComposite()) || $id === 'webform_multiple') {
      $interfaces[] = fn () => $this->types['WebformElementContainerBase'];
      $fields = [
        ...$fields,
        ...$this->elementContainer,
      ];
    }

    if ($plugin instanceof WebformElementOtherInterface) {
      $interfaces[] = fn () => $this->types['WebformElementOtherBase'];
      $fields = [
        ...$fields,
        ...$this->elementOtherBase,
      ];
    }

    if ($plugin instanceof OptionsBase) {
      $interfaces[] = fn () => $this->types['WebformElementOptionsBase'];
      $fields = [
        ...$fields,
        ...$this->elementOptions,
      ];
    }

    if ($plugin instanceof DateBase) {
      $interfaces[] = fn () => $this->types['WebformElementDateBase'];
      $fields = array_merge($fields, $this->elementDateBase);
    }

    if ($plugin instanceof WebformManagedFileBase) {
      $interfaces[] = fn () => $this->types['WebformElementManagedFileBase'];
      $fields = array_merge($fields, $this->elementManagedFileBase);
    }

    if ($plugin instanceof TextBase) {
      $interfaces[] = fn () => $this->types['WebformElementTextBase'];
      $fields = [
        ...$fields,
        ...$this->elementTextBase,
      ];
    }

    $typeName = EntitySchemaHelper::toPascalCase(['WebformElement', $id]);
    $this->types[$typeName] = new ObjectType([
      'name' => $typeName,
      'fields' => $fields,
      'description' => $description,
      'interfaces' => $interfaces,
    ]);
  }

}
